package com.jeeplus.sys.controller;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.jeeplus.aop.demo.annotation.DemoMode;
import com.jeeplus.aop.logging.annotation.ApiLog;
import com.jeeplus.core.excel.ExcelOptions;
import com.jeeplus.core.excel.ExportMode;
import com.jeeplus.core.excel.utils.EasyPoiUtil;
import com.jeeplus.sys.constant.enums.ProjectStatusEnum;
import com.jeeplus.sys.constant.enums.ProjectTypeEnum;
import com.jeeplus.sys.constant.enums.YesOrNoEnum;
import com.jeeplus.sys.domain.UserProject;
import com.jeeplus.sys.service.UserProjectService;
import com.jeeplus.sys.service.UserService;
import com.jeeplus.sys.service.dto.UserProjectDTO;
import com.jeeplus.sys.service.mapstruct.UserProjectWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.time.DateUtils;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;


/**
 * 项目Controller
 *
 * @author lizhenhao
 * @version 2022-08-02
 */
@Api("项目管理")
@RestController
@RequestMapping(value = "/sys/project")
public class UserProjectController {

    @Autowired
    private UserProjectService userProjectService;

    @Autowired
    private UserService userService;

    private int successNum;

    @ApiLog("查询项目列表")
    @PreAuthorize("hasAuthority('sys:project:list')")
    @GetMapping("list")
    public ResponseEntity<IPage<UserProject>> data(UserProject userProject, Page<UserProject> page) throws Exception {
        IPage<UserProject> result = userProjectService.selectPage (page,userProject);
        return ResponseEntity.ok (result);
    }

    /**
     * 查询项目数据
     * @param id
     * @return
     */
    @ApiLog("查询项目详情")
    @PreAuthorize ("hasAnyAuthority('sys:project:view','sys:project:add','sys:project:edit')")
    @GetMapping("queryById")
    public ResponseEntity queryById(@RequestParam("id") String id) {
        UserProject userProject = userProjectService.selectByProjectId ( id );
        return ResponseEntity.ok (userProject);
    }

    /**
     * 保存项目信息
     * @param projectDTO
     * @return
     */
    @ApiLog("保存项目")
    @PreAuthorize ("hasAnyAuthority('sys:post:add','sys:post:edit')")
    @PostMapping("save")
    public ResponseEntity save(@Valid @RequestBody UserProjectDTO projectDTO){
        //新增或编辑表单保存
        UserProject project = UserProjectWrapper.INSTANCE.toEntity (projectDTO);
        UserProject pro = userProjectService.projectDtoToDate(project,projectDTO);
        pro.setItemType(projectDTO.getItemType());

        if(StrUtil.isNotEmpty(projectDTO.getProjectHead())){
            String userId = userService.getUserIdByLogin(projectDTO.getProjectHead());
            if(StrUtil.isNotEmpty(userId)){
                pro.setProjectHead(userId);
            }else{
                return ResponseEntity.badRequest().body("项目负责人不存在，请重新输入");
            }

        }

        if(StrUtil.isNotEmpty(projectDTO.getEvaluationPersonOne())){
            String userId = userService.getUserIdByLogin(projectDTO.getEvaluationPersonOne());
            if(StrUtil.isNotEmpty(userId)){
                pro.setEvaluationPersonOne(userId);
            }else{
                return ResponseEntity.badRequest().body("签字评估师1不存在，请重新输入");
            }
        }

        if(StrUtil.isNotEmpty(projectDTO.getEvaluationPersonTwo())){
            String userId = userService.getUserIdByLogin(projectDTO.getEvaluationPersonTwo());
            if(StrUtil.isNotEmpty(userId)){
                pro.setEvaluationPersonTwo(userId);
            }else{
                return ResponseEntity.badRequest().body("签字评估师2不存在，请重新输入");
            }
        }

        if(StrUtil.isEmpty(projectDTO.getId())){

            //判断一下文号是否存在
            if(StrUtil.isNotEmpty(projectDTO.getDocumentNum())){
                if(ObjectUtil.isNotEmpty(userProjectService.selectByDocumentNum(projectDTO.getDocumentNum()))){
                    return ResponseEntity.badRequest().body("不可使用已存在的文号");
                }
            }

            userProjectService.save(pro);//新增
        }else{

            //判断一下文号是否存在
            if(StrUtil.isNotEmpty(projectDTO.getDocumentNum())){
                UserProject userProjects = userProjectService.selectByDocumentNum(projectDTO.getDocumentNum());
                if(ObjectUtil.isNotEmpty(userProjects)){
                    if(!projectDTO.getId().equals(userProjects.getId())){
                        return ResponseEntity.badRequest().body("不可使用已存在的文号");
                    }
                }
            }

            LambdaUpdateWrapper<UserProject> wrapper = userProjectService.getLambdaUpdateWrapper(projectDTO);
            userProjectService.update (pro,wrapper);//修改
        }

        return ResponseEntity.ok ("保存项目成功");
    }

    /**
     * 删除项目
     * @param ids
     * @return
     */
    @ApiLog("删除项目")
    @PreAuthorize ("hasAuthority('sys:project:del')")
    @DeleteMapping("delete")
    public ResponseEntity delete(String ids) {
        String idArray[] =ids.split(",");
        userProjectService.removeByIds (Lists.newArrayList (idArray));
        return ResponseEntity.ok ("删除项目成功");
    }

    /**
     * 导入项目数据
     *
     * @return
     */
    @DemoMode
    @PreAuthorize("hasAnyAuthority('sys:project:import')")
    @PostMapping("import")
    @ApiOperation(value = "导入项目数据excel")
    public ResponseEntity importFile(MultipartFile file) throws IOException{

        this.successNum = 0;

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        ArrayList<UserProject> arrayList = new ArrayList<>();
        HashMap<String,String> hashMap = new HashMap<>();

        HSSFWorkbook hssfWorkBook=new HSSFWorkbook(file.getInputStream());

        List<UserProjectDTO> listA = new ArrayList<>();
        List<UserProjectDTO> listB = new ArrayList<>();

        int sheetIndexAs = hssfWorkBook.getSheetIndex("项目明细表");
        if(sheetIndexAs != -1){
            //评估项目sheet
            listA = EasyPoiUtil.importSheetExcel(file, 1, 1, sheetIndexAs,1, UserProjectDTO.class);

            String resultA = importDecide(listA, sdf, arrayList, hashMap, successNum, ProjectTypeEnum.ASSESS.getValue());
            if(StrUtil.isNotEmpty(resultA)){
                return ResponseEntity.badRequest().body  (resultA);
            }
        }

        int sheetIndexCo = hssfWorkBook.getSheetIndex("咨询报告");
        if(sheetIndexCo != -1){
            //咨询项目sheet
            listB = EasyPoiUtil.importSheetExcel(file, 1, 1, sheetIndexCo,1, UserProjectDTO.class);

            String resultB = importDecide(listB, sdf, arrayList, hashMap, successNum, ProjectTypeEnum.CONSULTATION.getValue());
            if(StrUtil.isNotEmpty(resultB)){
                return ResponseEntity.badRequest().body  (resultB);
            }
        }

        if(sheetIndexAs == -1 && sheetIndexCo == -1) return ResponseEntity.badRequest().body  ("导入文件格式不正确，建议根据模板来进行数据的导入");


        if(hashMap.size() != listA.size() + listB.size()){
            return ResponseEntity.badRequest().body  ("文件中存在重复的文号或者多个空文号");
        }

        userProjectService.saveBatch(arrayList);

        return ResponseEntity.ok("已成功导入 " + successNum + " 条数据");
    }
    public String importDecide(List<UserProjectDTO> list,SimpleDateFormat sdf,ArrayList<UserProject> arrayList,HashMap<String,String> hashMap,int successNum,String itemType){

        for (UserProjectDTO project : list) {

            try {
                if(StrUtil.isNotEmpty(project.getEvaluationReportDateUi())){
                    String format = sdf.format(DateUtils.parseDate(project.getEvaluationReportDateUi(), new String[]{"yyyy/MM/dd"}));
                    project.setEvaluationReportDate(sdf.parse(format));
                    project.setStatus(ProjectStatusEnum.FINISH.getValue());
                }
            }catch (Exception e){
                if(StrUtil.isNotEmpty(project.getEvaluationReportDateUi())){
                    if(project.getEvaluationReportDateUi().contains("报告已作废")){
                        project.setStatus(ProjectStatusEnum.INVALID.getValue());
                        project.setAssessReportMessage(project.getEvaluationReportDateUi());
                    }else{
                        project.setStatus(ProjectStatusEnum.STAGING.getValue());
                        project.setAssessReportMessage(project.getEvaluationReportDateUi());
                    }
                }else{
                    project.setStatus(ProjectStatusEnum.STAGING.getValue());
                }
            }

            try {
                if(StrUtil.isNotEmpty(project.getEvaluationBaseDateUi())){
                    String format = sdf.format(DateUtils.parseDate(project.getEvaluationBaseDateUi(), new String[]{"yyyy/MM/dd"}));
                    project.setEvaluationBaseDate(sdf.parse(format));
                }
            }catch (Exception e){
                if(StrUtil.isNotEmpty(project.getEvaluationBaseDateUi()))
                    project.setAssessBaseMessage(project.getEvaluationBaseDateUi());
            }

            try {
                if(StrUtil.isNotEmpty(project.getInvoiceDateUi())){
                    String format = sdf.format(DateUtils.parseDate(project.getInvoiceDateUi(), new String[]{"yyyy/MM/dd"}));
                    project.setInvoiceDate(sdf.parse(format));
                }
            }catch (Exception e){
                if(StrUtil.isNotEmpty(project.getInvoiceDateUi()))
                    project.setInvoiceMessage(project.getInvoiceDateUi());
            }

            try {
                if(StrUtil.isNotEmpty(project.getReimbursementDateUi())){
                    String format = sdf.format(DateUtils.parseDate(project.getReimbursementDateUi(), new String[]{"yyyy/MM/dd"}));
                    project.setReimbursementDate(sdf.parse(format));
                }
            }catch (Exception e){
                e.printStackTrace();
            }

            //项目负责人、签字评估师1、签字评估师2 关联user表的id
            if(StrUtil.isNotEmpty(project.getProjectHead())){
                String id = userService.getUserIdByLogin(project.getProjectHead());
                if(StrUtil.isNotEmpty(id)){
                    project.setProjectHead(id);
                }else{
                    return "文件中有项目负责人无法找到，请修改为在档人员，或者不填写";
                }
            }

            if(StrUtil.isNotEmpty(project.getEvaluationPersonOne())){
                String id = userService.getUserIdByLogin(project.getEvaluationPersonOne());
                if(ObjectUtil.isNotEmpty(id)){
                    project.setEvaluationPersonOne(id);
                }else{
                    return "文件中有签字评估师无法找到，请修改为在档人员，或者不填写";
                }
            }

            if(StrUtil.isNotEmpty(project.getEvaluationPersonTwo())){
                String id = userService.getUserIdByLogin(project.getEvaluationPersonTwo());
                if(ObjectUtil.isNotEmpty(id)){
                    project.setEvaluationPersonTwo(id);
                }else{
                    return "文件中有签字评估师无法找到，请修改为在档人员，或者不填写";
                }
            }

            //文号的问题
            if(StrUtil.isNotEmpty(project.getDocumentNum())) {
                UserProject userProject = userProjectService.selectByDocumentNum(project.getDocumentNum());
                if(ObjectUtil.isNotEmpty(userProject)){
                    return "文件中存在已在档的文号";
                }
            }else{
                return "文件中存在空的文号，文号不允许为空";
            }

            //是否开票
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsInvoice())){
                project.setIsInvoice(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsInvoice())){
                project.setIsInvoice(YesOrNoEnum.NO.getValue());
            }

            //合同是否存档
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsContractArchive())){
                project.setIsContractArchive(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsContractArchive())){
                project.setIsContractArchive(YesOrNoEnum.NO.getValue());
            }

            //底稿是否完好
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsPapersIntact())){
                project.setIsPapersIntact(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsPapersIntact())){
                project.setIsPapersIntact(YesOrNoEnum.NO.getValue());
            }

            //底稿是否归档
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsPapersArchive())){
                project.setIsPapersArchive(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsPapersArchive())){
                project.setIsPapersArchive(YesOrNoEnum.NO.getValue());
            }

            //外勤是否已经报销
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsOpsReimbursement())){
                project.setIsOpsReimbursement(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsOpsReimbursement())){
                project.setIsOpsReimbursement(YesOrNoEnum.NO.getValue());
            }

            //是否已经报销提成
            if(YesOrNoEnum.YES.getLabel().equals(project.getIsCommissionReimbursement())){
                project.setIsCommissionReimbursement(YesOrNoEnum.YES.getValue());
            }else if(YesOrNoEnum.NO.getLabel().equals(project.getIsCommissionReimbursement())){
                project.setIsCommissionReimbursement(YesOrNoEnum.NO.getValue());
            }

            hashMap.put(project.getDocumentNum(),null);

            UserProject userProject = UserProjectWrapper.INSTANCE.toEntity(project);
            //项目类型 1.评估项目  2.咨询项目
            userProject.setItemType(itemType);
            arrayList.add(userProject);
            this.successNum++;
        }

        return null;
    }


    /**
     * 下载项目导入模板
     *
     * @param response
     * @return
     */
    @GetMapping("import/template")
    @PreAuthorize("hasAnyAuthority('sys:project:export')")
    @ApiOperation(value = "下载模板")
    public void importFileTemplate(HttpServletResponse response, HttpServletRequest request) {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("/dot/项目导入模板.xlsx");
            //强制下载不打开
            response.setContentType("application/force-download");
            OutputStream out = response.getOutputStream();
            //使用URLEncoder来防止文件名乱码或者读取错误
            response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode("project_template.xlsx", "UTF-8"));
            int b = 0;
            byte[] buffer = new byte[1000000];
            while (b != -1) {
                b = inputStream.read(buffer);
                if (b != -1) out.write(buffer, 0, b);
            }
            inputStream.close();
            out.close();
            out.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * 导出用户数据
     *
     * @param userProjectDTO
     * @param page
     * @param response
     * @throws Exception
     */
    @ApiLog("导出项目数据")
    @PreAuthorize("hasAnyAuthority('sys:project:export')")
    @GetMapping("export")
    @ApiOperation(value = "导出项目excel")
    public void exportFile(UserProjectDTO userProjectDTO, Page <UserProjectDTO> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = options.getFilename ( );
        UserProject project = UserProjectWrapper.INSTANCE.toEntity (userProjectDTO);
        Page <UserProject> userProjectPage = pageAToPageB(page);
        List <UserProject> result = new ArrayList<>();
        List <UserProjectDTO> userProjectDTOList = new ArrayList<>();
        if ( ExportMode.current.equals ( options.getMode ( ) ) ) {
            result = userProjectService.selectPage ( userProjectPage, project ).getRecords ( );
        } else {
            userProjectPage.setSize ( -1 );
            userProjectPage.setCurrent ( 0 );
            result = userProjectService.selectPage ( userProjectPage, project ).getRecords ( );
        }
        result.stream().forEach(item->{
            UserProjectDTO projectDTO = UserProjectWrapper.INSTANCE.toDTO(item);
            userProjectDTOList.add(projectDTO);
        });

        SimpleDateFormat formatter=new SimpleDateFormat("yyyy/MM/dd");
        List <UserProjectDTO> collect = userProjectDTOList.stream().map(item -> {
            return isChange(item,formatter);
        }).collect(Collectors.toList());
        if(ProjectTypeEnum.ASSESS.getValue().equals(userProjectDTO.getItemType()))
            EasyPoiUtil.exportExcel ( collect, "2021年资产评估项目备查登记表",  "项目明细表", UserProjectDTO.class, fileName, response );
        if(ProjectTypeEnum.CONSULTATION.getValue().equals(userProjectDTO.getItemType()))
            EasyPoiUtil.exportExcel ( collect, "2021年资产评估项目备查登记表",  "咨询报告", UserProjectDTO.class, fileName, response );
    }

    public Page<UserProject> pageAToPageB(Page<UserProjectDTO> page){

        Page<UserProject> pageB = new Page<>();
        pageB.setCurrent(page.getCurrent());
        pageB.setSize(page.getSize());
        pageB.setOrders(page.getOrders());

        return pageB;
    }

    public UserProjectDTO isChange(UserProjectDTO userProjectDTO,SimpleDateFormat formatter){

        if(StrUtil.isNotEmpty(userProjectDTO.getIsInvoice())){
            //是否开票
            if(YesOrNoEnum.YES.getValue().equals(userProjectDTO.getIsInvoice())){
                userProjectDTO.setIsInvoice(YesOrNoEnum.YES.getLabel());
            }else if(YesOrNoEnum.NO.getValue().equals(userProjectDTO.getIsInvoice())){
                userProjectDTO.setIsInvoice(YesOrNoEnum.NO.getLabel());
            }
        }


        if(StrUtil.isNotEmpty(userProjectDTO.getIsContractArchive())) {
            //合同是否存档
            if (YesOrNoEnum.YES.getValue().equals(userProjectDTO.getIsContractArchive())) {
                userProjectDTO.setIsContractArchive(YesOrNoEnum.YES.getLabel());
            } else if (YesOrNoEnum.NO.getValue().equals(userProjectDTO.getIsContractArchive())) {
                userProjectDTO.setIsContractArchive(YesOrNoEnum.NO.getLabel());
            }
        }

        if(StrUtil.isNotEmpty(userProjectDTO.getIsPapersIntact())) {
            //底稿是否完好
            if (YesOrNoEnum.YES.getValue().equals(userProjectDTO.getIsPapersIntact())) {
                userProjectDTO.setIsPapersIntact(YesOrNoEnum.YES.getLabel());
            } else if (YesOrNoEnum.NO.getValue().equals(userProjectDTO.getIsPapersIntact())) {
                userProjectDTO.setIsPapersIntact(YesOrNoEnum.NO.getLabel());
            }
        }

        if(StrUtil.isNotEmpty(userProjectDTO.getIsPapersArchive())) {
            //底稿是否归档
            if (YesOrNoEnum.YES.getValue().equals(userProjectDTO.getIsPapersArchive())) {
                userProjectDTO.setIsPapersArchive(YesOrNoEnum.YES.getLabel());
            } else if (YesOrNoEnum.NO.getValue().equals(userProjectDTO.getIsPapersArchive())) {
                userProjectDTO.setIsPapersArchive(YesOrNoEnum.NO.getLabel());
            }
        }

        if(StrUtil.isNotEmpty(userProjectDTO.getIsOpsReimbursement())) {
            //外勤是否已经报销
            if (YesOrNoEnum.YES.getValue().equals(userProjectDTO.getIsOpsReimbursement())) {
                userProjectDTO.setIsOpsReimbursement(YesOrNoEnum.YES.getLabel());
            } else if (YesOrNoEnum.NO.getValue().equals(userProjectDTO.getIsOpsReimbursement())) {
                userProjectDTO.setIsOpsReimbursement(YesOrNoEnum.NO.getLabel());
            }
        }

        if(StrUtil.isNotEmpty(userProjectDTO.getIsCommissionReimbursement())) {
            //是否已经报销提成
            if (YesOrNoEnum.YES.getValue().equals(userProjectDTO.getIsCommissionReimbursement())) {
                userProjectDTO.setIsCommissionReimbursement(YesOrNoEnum.YES.getLabel());
            } else if (YesOrNoEnum.NO.getValue().equals(userProjectDTO.getIsCommissionReimbursement())) {
                userProjectDTO.setIsCommissionReimbursement(YesOrNoEnum.NO.getLabel());
            }
        }


        //评估报告日
        if(ObjectUtil.isNotEmpty(userProjectDTO.getEvaluationReportDate())){
            String time=formatter.format(userProjectDTO.getEvaluationReportDate());
            userProjectDTO.setEvaluationReportDateUi(time);
        }else{
            if(StrUtil.isNotEmpty(userProjectDTO.getAssessReportMessage())){
                userProjectDTO.setEvaluationReportDateUi(userProjectDTO.getAssessReportMessage());
            }
        }
        //评估基准日
        if(ObjectUtil.isNotEmpty(userProjectDTO.getEvaluationBaseDate())){
            String time=formatter.format(userProjectDTO.getEvaluationBaseDate());
            userProjectDTO.setEvaluationBaseDateUi(time);
        }else{
            if(StrUtil.isNotEmpty(userProjectDTO.getAssessBaseMessage())){
                userProjectDTO.setEvaluationBaseDateUi(userProjectDTO.getAssessBaseMessage());
            }
        }
        //开票日期
        if(ObjectUtil.isNotEmpty(userProjectDTO.getInvoiceDate())){
            String time=formatter.format(userProjectDTO.getInvoiceDate());
            userProjectDTO.setInvoiceDateUi(time);
        }else{
            if(StrUtil.isNotEmpty(userProjectDTO.getInvoiceMessage())){
                userProjectDTO.setInvoiceDateUi(userProjectDTO.getInvoiceMessage());
            }
        }
        //报销日期
        if(ObjectUtil.isNotEmpty(userProjectDTO.getReimbursementDate())){
            String time=formatter.format(userProjectDTO.getReimbursementDate());
            userProjectDTO.setReimbursementDateUi(time);
        }



        return userProjectDTO;
    }


}
