package com.jeeplus.sys.service;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jeeplus.sys.domain.UserProject;
import com.jeeplus.sys.mapper.UserProjectMapper;
import com.jeeplus.sys.service.dto.UserProjectDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.UUID;

@Service
@Transactional
public class UserProjectService extends ServiceImpl<UserProjectMapper, UserProject> {

    @Autowired
    private UserProjectMapper userProjectMapper;

    public IPage<UserProject> selectPage(Page<UserProject> page, UserProject userProject){
        return userProjectMapper.selectPage(page, userProject);
    }

    public UserProject selectByProjectId(String projectId){
        return userProjectMapper.selectByProjectId(projectId);
    }

    public UserProject projectDtoToDate(UserProject project, UserProjectDTO projectDTO){
        SimpleDateFormat sdf = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss" );
        try {
            if(StrUtil.isNotEmpty(projectDTO.getEvaluationBaseDateUi()))
                project.setEvaluationBaseDate(sdf.parse(projectDTO.getEvaluationBaseDateUi()));

            if(StrUtil.isNotEmpty(projectDTO.getEvaluationReportDateUi()))
                project.setEvaluationReportDate(sdf.parse(projectDTO.getEvaluationReportDateUi()));

            if(StrUtil.isNotEmpty(projectDTO.getInvoiceDateUi()))
                project.setInvoiceDate(sdf.parse(projectDTO.getInvoiceDateUi()));

            if(StrUtil.isNotEmpty(projectDTO.getReimbursementDateUi()))
                project.setReimbursementDate(sdf.parse(projectDTO.getReimbursementDateUi()));

        } catch (Exception e) {
            e.printStackTrace();
        }
        return project;
    }

    public LambdaUpdateWrapper getLambdaUpdateWrapper(UserProjectDTO project){
        LambdaUpdateWrapper<UserProject> wrapper = new LambdaUpdateWrapper<UserProject>();

        if(StrUtil.isNotEmpty(project.getId()))
            wrapper.eq(UserProject::getId,project.getId());

        if(StrUtil.isEmpty(project.getEvaluationBaseDateUi()))
            wrapper.set(UserProject::getEvaluationBaseDate, null);

        if(StrUtil.isEmpty(project.getEvaluationReportDateUi()))
            wrapper.set(UserProject::getEvaluationReportDate, null);

        if(StrUtil.isEmpty(project.getInvoiceDateUi()))
            wrapper.set(UserProject::getInvoiceDate, null);

        if(StrUtil.isEmpty(project.getReimbursementDateUi()))
            wrapper.set(UserProject::getReimbursementDate, null);

        return wrapper;
    }

    public UserProject selectByDocumentNum(String documentNum){
        return userProjectMapper.selectByDocumentNum(documentNum);
    }
}
